@@ -66,7 +66,7 @@
 #define M_IMPLOSIVE   M_HOOK
 
 typedef struct {
-char *name;
+const char *name;
 int  flags;
 } ACCENTS;
 
@@ -419,6 +419,8 @@ void Translator::LookupAccentedLetter(unsigned int letter, char *ph_buf)
 	char ph_accent1[30];
 	char ph_accent2[30];
 
+	ph_accent2[0] = 0;
+
 	if((letter >= 0xe0) && (letter < 0x17f))
 	{
 		accent_data = letter_accents_0e0[letter - 0xe0];
@@ -548,7 +550,7 @@ void Translator::LookupLetter(unsigned int letter, int next_byte, char *ph_buf1)
 		single_letter[1] = ' ';
 		if(Lookup(&single_letter[2],ph_buf3) == 0)
 		{
-			TranslateRules(&single_letter[2], ph_buf3, sizeof(ph_buf3), NULL,0,NULL);
+			TranslateRules(&single_letter[2], ph_buf3, sizeof(ph_buf3), NULL,FLAG_NO_TRACE,NULL);
 		}
 	}
 
@@ -589,6 +591,7 @@ int Translator::TranslateLetter(char *word, char *phonemes, int control, int wor
 	int n_bytes;
 	int letter;
 	int len;
+	int save_option_phonemes;
 	char *p2;
 	char *pbuf;
 	char capital[20];
@@ -625,12 +628,15 @@ int Translator::TranslateLetter(char *word, char *phonemes, int control, int wor
 	}
 
 	if((ph_buf[0] == 0) && (translator_name != L('e','n')))
-//	if((ph_buf[0] == 0) && (word_length == 1) && (translator_name != L('e','n')))
 	{
 		// speak as English, check whether there is a translation for this character
 		SetTranslator2("en");
+		save_option_phonemes = option_phonemes;
+		option_phonemes = 0;
 		translator2->LookupLetter(letter, word[n_bytes], ph_buf);
 		SelectPhonemeTable(voice->phoneme_tab_ix);  // revert to original phoneme table
+		option_phonemes = save_option_phonemes;
+
 		if(ph_buf[0] != 0)
 		{
 			sprintf(phonemes,"%cen",phonSWITCH);
@@ -662,7 +668,10 @@ int Translator::TranslateLetter(char *word, char *phonemes, int control, int wor
 	}
 
 	len = strlen(phonemes);
-	sprintf(ph_buf2,"%c%s%s",0xff,capital,ph_buf);  // the 0xff marker will be removed or replaced in SetSpellingStress()
+	if(langopts.accents & 2)
+		sprintf(ph_buf2,"%c%s%s",0xff,ph_buf,capital);
+	else
+		sprintf(ph_buf2,"%c%s%s",0xff,capital,ph_buf);  // the 0xff marker will be removed or replaced in SetSpellingStress()
 	if((len + strlen(ph_buf2)) < N_WORD_PHONEMES)
 	{
 		strcpy(&phonemes[len],ph_buf2);
@@ -969,7 +978,11 @@ int Translator::LookupNum3(int value, char *ph_out, int suppress_null, int thous
 				LookupNum2(hundreds/10, x, ph_digits);
 			}
 
-			sprintf(ph_thousands,"%s%s%c",ph_digits,ph_10T,phonPAUSE_NOLINK);
+			if(langopts.numbers2 & 0x200)
+				sprintf(ph_thousands,"%s%s%c",ph_10T,ph_digits,phonPAUSE_NOLINK);  // say "thousands" before its number, not after
+			else
+				sprintf(ph_thousands,"%s%s%c",ph_digits,ph_10T,phonPAUSE_NOLINK);
+
 			hundreds %= 10;
 			if(hundreds == 0)
 				ph_100[0] = 0;
@@ -1262,7 +1275,10 @@ int Translator::TranslateNumber_1(char *word, char *ph_out, unsigned int *flags,
 	}
 
 	LookupNum3(value, ph_buf, suppress_null, thousandplex, prev_thousands);
-	sprintf(ph_out,"%s%s%s",ph_buf2,ph_buf,ph_append);
+	if((thousandplex > 0) && (langopts.numbers2 & 0x200))
+		sprintf(ph_out,"%s%s%s",ph_append,ph_buf2,ph_buf);  // say "thousands" before its number
+	else
+		sprintf(ph_out,"%s%s%s",ph_buf2,ph_buf,ph_append);
 
 
 	while(decimal_point)
