@@ -47,6 +47,7 @@
 #define OFFSET_CYRILLIC 0x420
 #define OFFSET_DEVANAGARI  0x900
 #define OFFSET_TAMIL  0xb80
+#define OFFSET_KOREAN 0x1100
 
 
 static const unsigned int replace_cyrillic_latin[] = 
@@ -108,7 +109,20 @@ Translator *SelectTranslator(const char *name)
 	switch(name2)
 	{
 	case L('a','f'):
-		tr = new Translator_Afrikaans();
+		{
+			static const short stress_lengths_af[8] = {170,140, 220,220,  0, 0, 250,270};
+			tr = new Translator();
+			SetupTranslator(tr,stress_lengths_af,NULL);
+
+			tr->langopts.stress_rule = 0;
+			tr->langopts.vowel_pause = 0x30;
+			tr->langopts.param[LOPT_DIERESES] = 1;
+			tr->langopts.param[LOPT_PREFIXES] = 1;
+			SetLetterVowel(tr,'y');  // add 'y' to vowels
+		
+			tr->langopts.numbers = 0x8d1 + NUM_ROMAN;
+			tr->langopts.accents = 1;
+		}
 		break;
 
 	case L('c','y'):   // Welsh
@@ -375,6 +389,7 @@ SetLengthMods(tr,3);  // all equal
 			tr->langopts.stress_rule = 2;
 			tr->langopts.numbers = 0x1009 + NUM_ROMAN;
 			tr->langopts.stress_flags =  0x6 | 0x10; 
+			tr->langopts.accents = 2;  // "capital" after letter name
 		}
 		break;
 
@@ -441,6 +456,24 @@ SetLengthMods(tr,3);  // all equal
 		}
 		break;
 
+	case L('k','o'):   // Korean, TEST
+		{
+			static const char ko_ivowels[] = {0x63,0x64,0x67,0x68,0x6d,0x72,0x74,0x75,0};  // y and i vowels
+			static const char ko_voiced[] = {0x02,0x05,0x06,0xab,0xaf,0xb7,0xbc,0};  // voiced consonants, l,m,n,N
+			tr = new Translator();
+
+			tr->letter_bits_offset = OFFSET_KOREAN;
+			memset(tr->letter_bits,0,sizeof(tr->letter_bits));
+			SetLetterBitsRange(tr,LETTERGP_A,0x61,0x75);
+			SetLetterBits(tr,LETTERGP_Y,ko_ivowels);
+			SetLetterBits(tr,LETTERGP_G,ko_voiced);
+
+			tr->langopts.stress_rule = 8;   // ?? 1st syllable if it is heavy, else 2nd syllable
+			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
+			tr->langopts.numbers = 0x0401;
+		}
+		break;
+
 	case L('k','u'):   // Kurdish
 		{
 			static const unsigned char stress_amps_ku[8] = {18,18, 20,20, 20,24, 24,22 };
@@ -582,6 +615,9 @@ SetLengthMods(tr,3);  // all equal
 			tr->langopts.stress_rule = 2;
 			tr->langopts.stress_flags = 0x16;
 			tr->langopts.length_mods0 = tr->langopts.length_mods;  // don't lengthen vowels in the last syllable
+
+			tr->langopts.numbers = 0x61 + 0x100000 + 0x4000;
+			tr->langopts.numbers2 = 0x200;  // say "thousands" before its number
 		}
 		break;
 
@@ -662,7 +698,7 @@ SetLengthMods(tr,3);  // all equal
 			SetupTranslator(tr,stress_lengths_ta,stress_amps_ta);
 			tr->langopts.length_mods0 = tr->langopts.length_mods;  // don't lengthen vowels in the last syllable
 
-			tr->langopts.stress_rule = 1;
+			tr->langopts.stress_rule = 0;
 			tr->langopts.stress_flags =  0x10004;   // use 'diminished' for unstressed final syllable
 			tr->letter_bits_offset = OFFSET_TAMIL;
 			tr->langopts.param[LOPT_WORD_MERGE] = 1;   // don't break vowels betwen words
@@ -1043,41 +1079,3 @@ if(prev->mnemonic == 'j')
 }
 #endif
 
-//**********************************************************************************************************
-
-
-
-Translator_Afrikaans::Translator_Afrikaans() : Translator()
-{//=========================================
-// Initialise options for this language
-
-	static const short stress_lengths2[8] = {170,140, 220,220,  0, 0, 250,270};
-	langopts.stress_rule = 0;
-	langopts.vowel_pause = 0x30;
-	langopts.param[LOPT_DIERESES] = 1;
-	langopts.param[LOPT_PREFIXES] = 1;
-	SetLetterVowel(this,'y');  // add 'y' to vowels
-
-	langopts.numbers = 0x8d1 + NUM_ROMAN;
-	langopts.accents = 1;
-	memcpy(stress_lengths,stress_lengths2,sizeof(stress_lengths));
-}
-
-
-int Translator_Afrikaans::TranslateChar(char *ptr, int prev_in, unsigned int c, unsigned int next_in, int *insert)
-{//===============================================================================================================
-// look for 'n  and replace by a special character (unicode: schwa)
-
-	if(!iswalpha(prev_in))
-	{
-		if((c == '\'') && (next_in == 'n'))
-		{
-			// n preceded by either apostrophe or U2019 "right single quotation mark"
-			ptr[0] = ' ';  // delete the  n
-			return(0x0259); // replace  '  by  unicode schwa character
-		}
-	}
-	return(c);
-}
-
-
