@@ -107,6 +107,8 @@ static const unsigned short brackets[] = {
 // other characters which break a word, but don't produce a pause
 static const unsigned short breaks[] = {'_', 0};
 
+// treat these characters as spaces, in addition to iswspace()
+static const wchar_t chars_space[] = {0x2500,0};  // box drawing horiz
 
 
 // Translate character codes 0xA0 to 0xFF into their unicode values
@@ -368,6 +370,9 @@ int IsAlpha(unsigned int c)
 	if((c >= 0x300) && (c <= 0x36f))
 		return(1);   // combining accents
 
+	if((c >= 0x1100) && (c <= 0x11ff))
+		return(1);  //Korean jamo
+
 	return(0);
 }
 
@@ -389,6 +394,13 @@ int IsDigit(unsigned int c)
 	return(0);
 }
 
+int IsSpace(unsigned int c)
+{//========================
+	if(wcschr(chars_space,c))
+		return(1);
+	return(iswspace(c));
+}
+
 
 Translator::Translator()
 {//=====================
@@ -864,7 +876,7 @@ if((wmark > 0) && (wmark < 8))
 				if((word_length == 1) && IsAlpha(wc))
 				{
 					posn = 0;
-					while(*wordx != ' ')
+					while((*wordx != ' ') && (*wordx != 0))
 					{
 						wordx += TranslateLetter(wordx, phonemes, 4, word_length);
 						posn++;
@@ -1332,7 +1344,7 @@ int Translator::TranslateWord2(char *word, WORD_TAB *wtab, int pre_pause, int ne
 	const char *new_language;
 	unsigned char bad_phoneme[4];
 	int word_copy_len;
-	char word_copy[N_WORD_BYTES];
+	char word_copy[N_WORD_BYTES+1];
 
 	len = wtab->length;
 	if(len > 31) len = 31;
@@ -1439,8 +1451,9 @@ int Translator::TranslateWord2(char *word, WORD_TAB *wtab, int pre_pause, int ne
 	}
 	else
 	{
+		int c2;
 		ix = 0;
-		while((word_copy[ix] = word[ix]) != ' ') ix++;
+		while(((c2 = word_copy[ix] = word[ix]) != ' ') && (c2 != 0) && (ix < N_WORD_BYTES)) ix++;
 		word_copy_len = ix;
 
 		flags = translator->TranslateWord(word, next_pause, wtab);
@@ -1471,22 +1484,25 @@ int Translator::TranslateWord2(char *word, WORD_TAB *wtab, int pre_pause, int ne
 				ok =0;
 			}
 
-			if(sylimit & 0x100)
+			if(ok != 0)
 			{
-				// only if the second word has $alt attribute
-				strcpy(ph_buf,word_phonemes);
-				flags2 = translator->TranslateWord(p2+1, 0, wtab+1);
-				if((flags2 & FLAG_ALT_TRANS) == 0)
+				if(sylimit & 0x100)
 				{
+					// only if the second word has $alt attribute
+					strcpy(ph_buf,word_phonemes);
+					flags2 = translator->TranslateWord(p2+1, 0, wtab+1);
+					if((flags2 & FLAG_ALT_TRANS) == 0)
+					{
+						ok = 0;
+						strcpy(word_phonemes,ph_buf);
+					}
+				}
+	
+				if((sylimit & 0x200) && ((wtab+1)->flags & FLAG_LAST_WORD))
+				{
+					// not if the next word is end-of-sentence
 					ok = 0;
-					strcpy(word_phonemes,ph_buf);
 				}
-			}
-
-			if((sylimit & 0x200) && ((wtab+1)->flags & FLAG_LAST_WORD))
-			{
-				// not if the next word is end-of-sentence
-				ok = 0;
 			}
 
 			if(ok)
@@ -1895,6 +1911,68 @@ int SubstituteChar(Translator *tr, unsigned int c, unsigned int next_in, int *in
 int Translator::TranslateChar(char *ptr, int prev_in, unsigned int c, unsigned int next_in, int *insert)
 {//=====================================================================================================
 	// To allow language specific examination and replacement of characters
+
+	int code;
+	int initial;
+	int medial;
+	int final;
+
+	static const unsigned char hangul_compatibility[0x34] = {
+	 0,  0x00,0x01,0xaa,0x02,0xac,0xad,0x03,
+	0x04,0x05,0xb0,0xb1,0xb2,0xb3,0xb4,0xb4,
+	0xb6,0x06,0x07,0x08,0xb9,0x09,0x0a,0xbc,
+	0x0c,0x0d,0x0e,0x0f,0x10,0x11,0x12,0x61,
+	0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,
+	0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,0x70,0x71,
+	0x72,0x73,0x74,0x75 };
+
+	switch(translator_name)
+	{
+	case L('a','f'):
+	// look for 'n  and replace by a special character (unicode: schwa)
+	
+		if(!iswalpha(prev_in))
+		{
+			if((c == '\'') && (next_in == 'n'))
+			{
+				// n preceded by either apostrophe or U2019 "right single quotation mark"
+				ptr[0] = ' ';  // delete the  n
+				return(0x0259); // replace  '  by  unicode schwa character
+			}
+		}
+		break;
+
+	case L('k','o'):
+		if(((code = c - 0xac00) >= 0) && (c <= 0xd7af))
+		{
+			// break a syllable hangul into 2 or 3 individual jamo
+			initial = (code/28)/21;
+			medial = (code/28) % 21;
+			final = code % 28;
+
+			if(initial == 11)
+			{
+				// null initial
+				c = medial + 0x1161;
+				if(final > 0)
+					*insert = final + 0x11a7;
+			}
+			else
+			{
+				// extact the initial and insert the remainder with a null initial
+				c = initial + 0x1100;
+				*insert = (11*28*21) + (medial*28) + final + 0xac00;
+			}
+			return(c);
+		}
+		else
+		if(((code = c - 0x3130) >= 0) && (code < 0x34))
+		{
+			// Hangul compatibility jamo
+			return(hangul_compatibility[code] + 0x1100);
+		}
+		break;
+	}
 	return(SubstituteChar(this,c,next_in,insert));
 }
 
@@ -1995,10 +2073,9 @@ void *Translator::TranslateClause(FILE *f_text, const void *vp_input, int *tone_
 		}
 	}
 
+	memset(&ph_list2[0],0,sizeof(ph_list2[0]));
 	ph_list2[0].phcode = phonPAUSE_SHORT;
-   ph_list2[0].stress = 0;
-	ph_list2[0].tone_number = 0;
-	ph_list2[0].sourceix = 0;
+
 	n_ph_list2 = 1;
 	prev_last_stress = 0;
 	prepause_timeout = 0;
@@ -2167,7 +2244,7 @@ if((c == '/') && (langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(prev_ou
 				next_in = char_inserted;
 
 			// allow certain punctuation within a word (usually only apostrophe)
-			if(!IsAlpha(c) && !iswspace(c) && (wcschr(punct_within_word,c) == 0))
+			if(!IsAlpha(c) && !IsSpace(c) && (wcschr(punct_within_word,c) == 0))
 			{
 				if(IsAlpha(prev_out))
 				{
@@ -2345,7 +2422,7 @@ if((c == '/') && (langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(prev_ou
 					}
 					else
 					{
-						if(iswspace(prev_out))
+						if(IsSpace(prev_out))
 							single_quoted = 1;
 						else
 							single_quoted = 0;
@@ -2388,7 +2465,7 @@ if((c == '/') && (langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(prev_ou
 			}
 		}
 
-		if(iswspace(c))
+		if(IsSpace(c))
 		{
 			if(space_inserted)
 			{
